package com.muyu.rule.server.controller;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.muyu.common.core.domain.Result;
import com.muyu.rule.common.domain.DataValue;
import com.muyu.rule.common.domain.RuleEngineVersion;
import com.muyu.rule.common.domain.req.VersionAddReq;
import com.muyu.rule.common.utils.OssUtil;
import com.muyu.rule.server.EngineConfig;
import com.muyu.rule.server.basic.BasicEngine;
import com.muyu.rule.server.complie.SourceCodeComplier;
import com.muyu.rule.server.load.JavaBinaryClassLoader;
import com.muyu.rule.server.service.RuleEngineVersionService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.tools.JavaCompiler;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Map;

/**
 * @Author：张承志
 * @Package：com.muyu.rule.server.controller
 * @Project：cloud-etl-rule
 * @name：RuleEngineVersionController
 * @Date：2024/8/25 10:41
 */
@Log4j2
@RestController
@RequestMapping("/version")
@RequiredArgsConstructor
@Tag(name = "规则版本控制层", description = "规则版本管理,查看等相关操作")
public class RuleEngineVersionController {


    /**
     * 保存代码
     * @param ruleEngineVersion
     * @return
     */


    @PostMapping("/saveCode")
    public Result saveCode(@RequestBody RuleEngineVersion ruleEngineVersion){

        UpdateWrapper<RuleEngineVersion> updateWrapper = new UpdateWrapper<>();

        updateWrapper.eq(ruleEngineVersion.getRuleId(),ruleEngineVersion.getVersionClazz());


     versionService.Upload(ruleEngineVersion.getVersionClazz(),ruleEngineVersion.getClassName());


        return   Result.success();
    }


    @Autowired
    private RuleEngineVersionService versionService;

    /**
     * 添加规则引擎版本
     * @param versionAddReq 引擎版本添加实体类
     * @return
     */
 @PostMapping("/insertVersion")
 @Operation(summary = "增加规则引擎版本",description = "通过传入版本添加的实体类,进行规则引擎的版本添加")
    public Result insertVersion(@Validated @RequestBody VersionAddReq versionAddReq){

     boolean save = versionService.save(RuleEngineVersion.addBuild(versionAddReq));
     if (true==save){
      //添加成功则把增加的代码进行java编译,进行热加载
         versionService.HotLoadClass(versionAddReq.getClassName(),versionAddReq.getVersionClazz());
     }

     return  Result.success();
 }

    /**
     * 激活引擎请求
     * @param id
     * @return
     */
    @PostMapping("/activate/{id}")
    @Operation(summary = "规则引擎的激活", description = "通过Id激活规则引擎")
    public Result activate(@PathVariable("id") Long id){

        versionService.activate(id);

        return Result.success();
    }

    /**
     * 取消引擎请求
     */

    @PostMapping("/disable/{id}")
    @Operation(summary = "规则引擎的取消", description = "通过Id取消规则引擎")
    public Result disable(@PathVariable("id") Long id){

        versionService.disable(id);

        return Result.success();
    }

    /**
     * 将源码写入到OSS中
     * @param
     * @return
     */

    @PostMapping("/deposit")
    public Result deposit(@RequestBody VersionAddReq ver ){

        log.info("传入参数{}"+ver);

        String versionClazz = ver.getVersionClazz();
        String className = ver.getClassName();

        versionService.Upload(versionClazz,className);

        return Result.success();
    }

@PostMapping("/readByOss/{className}")
    public Result readByOss(@PathVariable String className){

Object  o=  versionService.readByOss(className);

    return Result.success(o);
    }

    @PostMapping("/testvalue/{versionClazz}")
public Result testValue(@PathVariable("versionClazz") String versionClazz,@RequestBody DataValue dataValue){

        versionService.testValue(versionClazz,dataValue);

        return Result.success();
}




















@PostMapping("/test")
public Result test(){

    String filePath = "cloud-rule-server/src/main/java/com/muyu/rule/server/basic/engine/value/";

    String pack = "com.muyu.rule.server.basic.engine.value.";

    File file = new File(filePath+"ENGINE_phone_zzzzz_V9.java");

    String ossFilePath = "ENGINE_phone_zzzzz_V9.java";

    try {
        boolean newFile = file.createNewFile();

        String string = OssUtil.readFileContentFromOSS("zcz-vfd-1000",ossFilePath);
        System.out.println(string);
        try (FileWriter writer = new FileWriter(file)) {
            writer.write(string);
        }catch (IOException e) {
            // 处理异常，例如记录日志或者通知用户
            e.printStackTrace();
        }

        //扫描原码进行编译+-+-
        SourceCodeComplier.javaCompilerPath(filePath);
//对class文件进行自定义类加载规则引擎
        Map<String ,Class<?>> stringClassMap  = JavaBinaryClassLoader.loadClassesByLocation(pack,filePath);

    } catch (IOException e) {
        throw new RuntimeException(e);
    }

    // stringClassMap.forEach(EngineContainer::loadEngineInstance);

    return Result.success();
}
@PostMapping("/look")
public Result look (){

    String ossFilePath = "ENGINE_phone_zzzzz_V9.java";

    String filePath ="home/"+ossFilePath;

    OssUtil.downloadFileForBucket("zcz-vfd-1000",ossFilePath,filePath);

    EngineConfig engineConfig = new EngineConfig();

    SourceCodeComplier.javaCompilerPath(engineConfig.getLocation());

    //对class文件进行自定义类加载规则引擎
    Map<String ,Class<?>> stringClassMap  = JavaBinaryClassLoader.loadClassesByLocation(engineConfig.getPack(),engineConfig.getLocation());

    return Result.success();
}

}



