package com.muyu.rule.server.service.impl;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.enums.SystemYesNo;
import com.muyu.rule.common.domain.DataValue;
import com.muyu.rule.common.domain.RuleEngine;
import com.muyu.rule.common.domain.RuleEngineVersion;
import com.muyu.rule.common.utils.OssUtil;
import com.muyu.rule.server.EngineConfig;
import com.muyu.rule.server.basic.BasicEngine;
import com.muyu.rule.server.complie.SourceCodeComplier;
import com.muyu.rule.server.load.JavaBinaryClassLoader;
import com.muyu.rule.server.mapper.RuleEngineVersionMapper;
import com.muyu.rule.server.scan.JavaCodeScan;
import com.muyu.rule.server.service.RuleEngineVersionService;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.nio.charset.StandardCharsets;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Base64;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import static com.muyu.rule.server.constant.SuffixClass.Suffix_JAVA;

/**
 * @Author：张承志
 * @Package：com.muyu.rule.server.service.impl
 * @Project：cloud-etl-rule
 * @name：RuleEngineServiceImpl
 * @Date：2024/8/25 14:17
 */
@Log4j2
@Service
public class RuleEngineServiceImpl extends ServiceImpl<RuleEngineVersionMapper, RuleEngineVersion> implements RuleEngineVersionService {

    @Override
    public List<RuleEngineVersion> selectRuleEngineVersion(Long id) {

        LambdaQueryWrapper<RuleEngineVersion> queryWrapper = new LambdaQueryWrapper<>();

        queryWrapper.eq(
                RuleEngineVersion::getRuleId, id);

        List<RuleEngineVersion> versionList = this.list(queryWrapper);

        return versionList;
    }

    @Override
    public void activate(Long id) {
        this.updateOpen(id, SystemYesNo.YES.getCode());
    }

    @Override
    public void disable(Long id) {
        this.updateOpen(id, SystemYesNo.NO.getCode());
    }



    /**
     * bucket名称
     *
     * @return
     */
    private static String bucketName = "zcz-vfd-1000";

    /**
     * 上传
     * @param versionClazz java源代码
     * @param className    类的名
     */
    @Override
    public void Upload(String versionClazz, String className) {

        /**
         * 将java源码存入OSS
         */
        ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(versionClazz.getBytes());
        OssUtil.uploadFileInputStreamForBucket(bucketName, className + Suffix_JAVA, byteArrayInputStream);

    }

    @Override
    public Object readByOss(String className) {

        String string = OssUtil.readFileContentFromOSS(bucketName, className);

        return string;
    }

    @Override
    public void HotLoadClass(String className, String versionClazz) {

        String ossFilePath = className + Suffix_JAVA;

        EngineConfig engineConfig = new EngineConfig();

        //  String filePath ="home/"+ossFilePath;
        String filePath = engineConfig.getServerPath() + ossFilePath;
//下载容器
        OssUtil.downloadFileForBucket(bucketName, ossFilePath, filePath);
//编译
        SourceCodeComplier.javaCompilerPath(engineConfig.getServerPath());

        //对class文件进行自定义类加载规则引擎
        Map<String, Class<?>> stringClassMap = JavaBinaryClassLoader.loadClassesByLocation(engineConfig.getServerPack(), engineConfig.getServerPath());
    }

    static Map<String, BasicEngine<DataValue>> engineMap = new ConcurrentHashMap<>();

    static {
        try {
            Class<?> aClass = Class.forName("com.muyu.rule.server.basic.engine.value.ENGINE_VALUE_VFD1000_V1");
            Class<?> aClass2 = Class.forName("com.muyu.rule.server.basic.engine.value.ENGINE_VALUE_VFD1000_V2");
            try {
                engineMap.put("ENGINE_VALUE_VFD1000_V1", (BasicEngine<DataValue>) aClass.newInstance());
                engineMap.put("ENGINE_VALUE_VFD1000_V2", (BasicEngine<DataValue>) aClass2.newInstance());
            } catch (InstantiationException e) {
                throw new RuntimeException(e);
            } catch (IllegalAccessException e) {
                throw new RuntimeException(e);
            }
        } catch (ClassNotFoundException e) {
            throw new RuntimeException(e);
        }

    }

    @Override
    public void testValue(String versionClazz, DataValue dataValue) {

        BasicEngine<DataValue> valueBasicEngine = engineMap.get(versionClazz);

        valueBasicEngine.set(dataValue);

        valueBasicEngine.execution();

    }

    /**
     * 修改激活
     *
     * @param id
     * @param open
     */
    public void updateOpen(Long id, String open) {

        UpdateWrapper<RuleEngineVersion> updateWrapper = new UpdateWrapper<>();

        updateWrapper.eq("id", id).set("open", open);

        this.update(updateWrapper);
    }
}
